SystemOrganization addCategory: #Moe!

MAContainerComponent subclass: #MOEmbeddedContainerComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Moe'!

!MOEmbeddedContainerComponent methodsFor: 'actions' stamp: 'jmck 6/9/2009 11:42'!
save
	self validate ifFalse: [ ^ self ].
	self commit.
	self model save.
	self answer: self model.! !

!String methodsFor: '*moe' stamp: 'jmck 6/9/2009 10:31'!
asIntegerArray
	^(self allButFirst allButLast subStrings: #($,)) 
			collect: [ :string | Number readFrom: (ReadStream on: string) ]! !

MACheckedMemento subclass: #MOEmbeddedMemento
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Moe'!

!MOEmbeddedMemento methodsFor: 'private' stamp: 'jmck 6/8/2009 21:00'!
pullRaw
	| result |
	result := Dictionary new.
	self description do: [ :each |
		each isEmbedded
			ifTrue: [ result at: each put: (each accessor next read: self model) ]
			ifFalse: [ 
				each isEmbeddedCollection 
					ifTrue: [result at: each put: (self model perform: each accessor selector) ]
					ifFalse: [ result at: each put: (self model readUsing: each) ] ] ].
	^ result! !

MAPriorityContainer subclass: #MOEmbeddedContainer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Moe'!

!MOEmbeddedContainer commentStamp: 'jmck 6/11/2009 18:34' prior: 0!
I exist to point to an MOEmbeddedContainerComponent which saves the model to the DB after commiting the memento to the model!

!MOEmbeddedContainer methodsFor: 'accessing' stamp: 'jmck 6/9/2009 19:19'!
componentClass
	^MOEmbeddedContainerComponent! !

MAChainAccessor subclass: #MOEmbeddedAccessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Moe'!

!MOEmbeddedAccessor commentStamp: 'jmck 6/9/2009 21:26' prior: 0!
I am a ChainAccessor implementing an accessor strategy for an embedded (child) MOEmbeddedObjects. 
I read: the *id* of the object (mainly so that MARelation will write the id to the database field, instead of the object ;) 
I write:to: a model the described object (so that the model object stores my described object rather than just my object's id, natch)

Note: The read: behavior is wrong for MAMemento>>pullRaw. See MOEmbeddedMemento which overrides pullRaw to accomodate this behavior!

!MOEmbeddedAccessor class methodsFor: 'instance-creation' stamp: 'jmck 6/11/2009 16:26'!
mappedTo: aSelector
	^self on: aSelector accessor: #id asAccessor! !

!MOEmbeddedAccessor methodsFor: 'as yet unclassified' stamp: 'jmck 6/11/2009 19:33'!
canRead: aModel
	^ aModel notNil and: [ super canRead: aModel ]! !

!MOEmbeddedAccessor methodsFor: 'model' stamp: 'jmck 6/7/2009 01:05'!
write: anObject to: aModel
	self next write: anObject to: aModel! !

MASelectorAccessor subclass: #MOEmbeddedCollectionAccessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Moe'!

!MOEmbeddedCollectionAccessor methodsFor: 'model' stamp: 'jmck 6/11/2009 16:32'!
read: aModel
	"Returns a postgresql string representation of an integer array"	
	^String streamContents: [:strm |
		strm nextPut: ${.
		(super read: aModel) do: [ :each | 
			strm nextPutAll: each id asString ] separatedBy: [ strm nextPut: $, ].
		strm nextPut: $} ].
	! !

!MADescription methodsFor: '*moe' stamp: 'jmck 6/8/2009 20:59'!
beEmbedded
	self propertyAt: #embedded put: true! !

!MADescription methodsFor: '*moe' stamp: 'jmck 6/8/2009 20:59'!
beEmbeddedCollection
	self propertyAt: #embeddedCollection put: true! !

!MADescription methodsFor: '*moe' stamp: 'jmck 6/11/2009 19:51'!
embeddedAccessor: aSelector
	self accessor: (MOEmbeddedAccessor mappedTo: aSelector);
		beEmbedded! !

!MADescription methodsFor: '*moe' stamp: 'jmck 6/11/2009 19:52'!
embeddedCollectionAccessor: aSelector
	self accessor: (MOEmbeddedCollectionAccessor selector: aSelector); beEmbeddedCollection! !

!MADescription methodsFor: '*moe' stamp: 'jmck 6/8/2009 20:59'!
isEmbedded
	^self propertyAt: #embedded ifAbsent: [ false ]! !

!MADescription methodsFor: '*moe' stamp: 'jmck 6/8/2009 20:59'!
isEmbeddedCollection
	^self propertyAt: #embeddedCollection ifAbsent: [ false ]! !

MAMappedObject subclass: #MOEmbeddedObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Moe'!

!MOEmbeddedObject commentStamp: 'jmck 6/11/2009 19:26' prior: 0!
Moe is a simple object persistency strategy using Magritte and Roe - Moe!!
The main architectural feature is that Moe make the database "mirror" the Smalltalk objects rather than force the objects to adhere to all sorts of strangulations like pesky foreign key definitions and the like. 

The base class, MOEmbeddedObject,  is a descendant of MAMappedObject. All persisting objects must descend from MOEmbeddedObject and override loadEmbeddedObjectFor: aDescription at: anId. MODatalistObject and MODatabaseObject are two classes from which you can descend "out of the box". Datalists keep a list of ALL of its instances in a class variable. The latter loads only the objects as they are referenced during. 


Describe an embedded object Customer like so:
descriptionCustomer
	^MAToOneRelationDescription new
		classes: (Array with: Customer); 
		embeddedAccessor: #customer;
		label: 'Customer';
		sqlColumnName: 'customer';
		sqlColumnType: 'integer';
		beEmbedded;
		priority: 10;
		yourself
		
When a Magritte description identifies itself as embedded (using beEmbedded or beEmbeddedCollection that object will have its ID stored in the database in the column where the object should be, and on retrieval from the DB it will be restored with the Smalltalk object. Collections will have an array of integers stored in the DB from which the collection will be restored upon retrieval.

Of course, embedded objects must descend from MOEmbeddedObject. See MODatalistObject and MODatabaseObject which override loadEmbeddedObjectFor: aDescription withId: anId.

This is pretty lightweight, and is meant only as a means of persistency (read: data backup and restore) 
!

MOEmbeddedObject subclass: #MODatabaseObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Moe'!

!MODatabaseObject commentStamp: 'jmck 6/11/2009 18:00' prior: 0!
I am like the Datalist object but I don't load all of the insances of me from the database.!

!MODatabaseObject methodsFor: 'persistence' stamp: 'jmck 6/9/2009 19:38'!
loadEmbeddedObjectFor: aDescription at: anId
	"This can be overridden if you would rather hit the database for each embedded object
	The current implementation uses uses a class variable to store ALL objects from the DB"
	
	 ^ aDescription classes first loadId: anId! !

MOEmbeddedObject subclass: #MODatalistObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Moe'!

MODatalistObject class
	instanceVariableNames: 'List'!

!MODatalistObject commentStamp: 'jmck 6/11/2009 17:58' prior: 0!
I am really an example class of one way to maintain the database of objects in the image. I load all the instances of me in a List variable!

MODatalistObject class
	instanceVariableNames: 'List'!

!MODatalistObject class methodsFor: 'persistency' stamp: 'jmck 6/11/2009 09:22'!
list
	
	^List ifNil: 
	
		"["
			 [ (List := OrderedCollection new) addAll: self loadAll ] 
		
		
		"on: Error do: [ :ex | List := nil ] ]"! !

!MODatalistObject methodsFor: 'private-sql' stamp: 'jmck 6/9/2009 19:46'!
insert
	super insert.
	self list add: self.! !

!MODatalistObject methodsFor: 'persistence' stamp: 'jmck 6/9/2009 19:36'!
list
	^self class list! !

!MODatalistObject methodsFor: 'persistence' stamp: 'jmck 6/11/2009 18:21'!
loadEmbeddedObjectFor: aDescription at: anId
	^ aDescription classes first list detect: [ :each | each id = anId ] ifNone: [ nil ]! !

!MODatalistObject methodsFor: 'persistence' stamp: 'jmck 6/11/2009 18:35'!
loadEmbeddedObjectFor: aDescription withId: anId
	^ aDescription classes first list detect: [ :each | each id = anId ] ifNone: [ nil ]! !

!MOEmbeddedObject class methodsFor: 'as yet unclassified' stamp: 'jmck 6/9/2009 19:19'!
descriptionContainer
	^MOEmbeddedContainer new! !

!MOEmbeddedObject class methodsFor: 'as yet unclassified' stamp: 'jmck 6/8/2009 20:44'!
load: aRelation
	^ (super load: aRelation) do: [ :each | each loadEmbedded ].
	! !

!MOEmbeddedObject class methodsFor: 'as yet unclassified' stamp: 'jmck 6/9/2009 19:40'!
loadId: anId
	^(super loadId: anId) loadEmbedded! !

!MOEmbeddedObject methodsFor: 'persistence' stamp: 'jmck 6/11/2009 17:45'!
loadEmbedded
	self description do: [ :desc |
		desc isEmbedded 
			ifTrue: [ self loadEmbeddedDescription: desc ]
			ifFalse: [
				desc isEmbeddedCollection 
					ifTrue: [ 
						self loadEmbeddedCollectionDescription: desc ]]].
! !

!MOEmbeddedObject methodsFor: 'persistence' stamp: 'jmck 6/11/2009 18:35'!
loadEmbeddedCollectionDescription: aDescription
	| ids embeddedObjects |	
	embeddedObjects := OrderedCollection new.
	ids := self perform: aDescription accessor selector.
	(ids notNil and: [ ids size > 2 ]) 
			ifTrue: [
				(ids asIntegerArray)
					inject: embeddedObjects
					into: [ :collection :i | 
						collection add: (self loadEmbeddedObjectFor: aDescription withId: i); yourself]].
	aDescription accessor write: embeddedObjects to: self ! !

!MOEmbeddedObject methodsFor: 'persistence' stamp: 'jmck 6/11/2009 18:35'!
loadEmbeddedDescription: aDescription
	| embeddedid |
	(embeddedid := self perform: aDescription accessor next selector) isNil
		ifFalse: [
			aDescription accessor
				write: (self loadEmbeddedObjectFor: aDescription withId: embeddedid)
				to: self ]! !

!MOEmbeddedObject methodsFor: 'persistence' stamp: 'jmck 6/9/2009 19:35'!
loadEmbeddedObjectFor: aDescription at: anId
	"This can be overridden if you would rather hit the database for each embedded object
	The current implementation uses uses a class variable to store ALL objects from the DB"
	
	 self subclassResponsibility! !

!MOEmbeddedObject methodsFor: 'persistence' stamp: 'jmck 6/11/2009 18:35'!
loadEmbeddedObjectFor: aDescription withId: anId
	"This can be overridden if you would rather hit the database for each embedded object
	The current implementation uses uses a class variable to store ALL objects from the DB"
	
	 self subclassResponsibility! !

!MOEmbeddedObject methodsFor: 'persistence' stamp: 'jmck 6/9/2009 19:17'!
mementoClass
	^MOEmbeddedMemento! !
